#include "globals.hh"

// conversion between Symbols (words, strings) and ids
hash_map<Symbol, unsigned int, symbol_hash> symbolid;
vector<Symbol> id2symbol;

Symbol char2symbol(char c)
{
  char str[2] = " ";
  str[0] = c;
  return Symbol(str);
}

const unsigned int deque_hash::numprimes = 10;
const unsigned int deque_hash::primes[] = {
  1001303, 40032037, 3025669, 9385469, 2484707, 
  5636153, 5636153, 5636153, 47282951, 47282951
};

SymbolID getSymbolID(Symbol const &s)
{
  hash_map<Symbol, unsigned int, symbol_hash>::iterator 
    iter = symbolid.find(s);
  
  // no id for this word yet, add one, return it.
  if (iter == symbolid.end())
    {
      SymbolID newid = id2symbol.size();
      symbolid[s] = newid;
      id2symbol.push_back(s);
      return newid;
    }
  else
    // we already have an id.
    return iter->second;
}

ostream &operator<<(ostream & os, deque<SymbolID> const &dq)
{
  copy(dq.begin(), dq.end(), ostream_iterator<SymbolID>(os, " "));

  return os;
}

void print_deque_readable(deque<SymbolID> const &dq, ostream &os)
{
  for (deque<SymbolID>::const_iterator i = dq.begin(); i != dq.end(); i++)
    {
      if (*i < id2symbol.size())
	os << id2symbol[*i] << ' ';
      else
	os << "##undef## ";
    }
}

